import serial
import serial.tools.list_ports as list_ports
import sys
import os
import cv2
import gui
import subprocess


class MovingRailServer:
    min_speed = 2.0
    max_speed = 90.0

    def __init__(self, window):
        self.port = None
        self.motor_on = False
        self.recv_buff = ""
        self.window = window

    def scan_devices(self):
        dev = [x.device for x in list_ports.comports()]
        return dev

    def connect(self, device, baudrate, timeout=1):
        try:
            self.port = serial.Serial(device, baudrate, timeout=timeout)
            self.port.send_break()
        except serial.SerialException as e:
            self.window.display_msg("Error: {}".format(e), gui.MSG_TYPE.SERIAL)
            return False
        return True

    def is_connected(self):
        return self.port is not None

    def close(self):
        self.port.close()
        self.port = None

    def motor_enabled(self, state):
        cmd = "M17" if state else "M18"
        success = self._write(cmd)
        if success:
            self.motor_on = state
        return success

    def home(self):
        cmd = "G28"
        success = self._write(cmd)
        return success

    def move(self, pos, feedrate=10):
        cmd = "G1 X{} F{}".format(pos, feedrate)
        success = self._write(cmd)
        return success

    def _write(self, data, add_break=True):
        w = self.window
        if self.port is None:
            w.display_msg("Error: no active serial.", gui.MSG_TYPE.CONTROL)
            return False
        if add_break:
            data += "\n"
        data_b = bytes(data, encoding="ascii")
        self.port.write(data_b)
        w.append_gcode("send: {}".format(data.strip()))
        return True

    def update_read(self):
        w = self.window
        if self.port is None:
            return
        n = self.port.in_waiting
        if n == 0:
            return
        ans_bytes = self.port.read(n)
        try:
            ans_txt = str(ans_bytes, encoding="ascii")
        except UnicodeDecodeError:
            ans_txt = "?\n"
        self.recv_buff += ans_txt
        lines = self.recv_buff.split("\n")
        if len(lines) == 1:
            return
        for txt in lines[:-1]:
            w.append_gcode("recv: {}".format(txt.strip()))
        self.recv_buff = lines[-1]


class CameraServer:
    known_ext = (".jpg", ".png")

    def __init__(self, window):
        self.window = window
        self.process = None
        self.folder = "."

    def shoot(self):
        if self.process is None:
            cmd = r"C:\Program Files (x86)\digiCamControl\CameraControlCmd.exe /capture /folder {}"
            self.process = subprocess.Popen(cmd.format(self.folder))
            self.window.display_msg("Shooting...", gui.MSG_TYPE.CAMERA)
        else:
            self.window.display_msg("Error: shooting ongoing.", gui.MSG_TYPE.CAMERA)

    def set_folder(self, folder):
        self.folder = folder

    def get_progress(self):
        if self.process is None:
            return False
        poll = self.process.poll()
        if poll is None:
            # still running
            return False
        self.process = None
        self.window.display_msg("Shooting done.", gui.MSG_TYPE.CAMERA)
        return True

    def is_known_ext(self, txt):
        known = False
        for ext in self.known_ext:
            if txt.lower().endswith(ext):
                known = True
                break
        return known

    def get_last_img(self):
        files = [os.path.join(self.folder, x) for x in os.listdir(self.folder)
                 if self.is_known_ext(x)]
        if len(files) == 0:
            return None
        files.sort(key=os.path.getctime)
        return files[-1]

    def close(self):
        pass
